# Generated by roxygen2: do not edit by hand

export(BioIndex)
export(LFD)
export(MEDITS.distance)
export(MEDITS.to.dd)
export(MIW)
export(aggregate_gsas)
export(bubble_plot_by_haul_indexes)
export(bubbleplot_RS_by_hauls)
export(check_date_haul)
export(check_dictionary)
export(check_hauls_TBTA)
export(check_numeric_range)
export(convert_coordinates)
export(dd.distance)
export(dd.to.MEDITS)
export(hauls_position)
export(index_on_grid)
export(index_recr)
export(index_spawn)
export(index_ts_F)
export(index_ts_M)
export(indices_ts)
export(merge_TATB)
export(merge_TATBTC)
export(merge_TATC)
export(overlayGrid)
export(run_BioIndex_app)
export(sex_ratio)
export(sex_ratio_on_grid)
export(spear)
export(spearman)
import(ggplot2)
import(grDevices)
import(graphics)
import(mgcv)
importFrom(dplyr,arrange)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_map)
importFrom(ggplot2,coord_sf)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_contour)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,map_data)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_size)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(grDevices,dev.copy)
importFrom(grDevices,dev.off)
importFrom(grDevices,heat.colors)
importFrom(grDevices,jpeg)
importFrom(grDevices,tiff)
importFrom(graphics,box)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(gridExtra,grid.arrange)
importFrom(hms,hms)
importFrom(magrittr,"%>%")
importFrom(marmap,as.xyz)
importFrom(marmap,getNOAA.bathy)
importFrom(methods,is)
importFrom(mgcv,gam)
importFrom(reshape2,dcast)
importFrom(reshape2,melt)
importFrom(shiny,runApp)
importFrom(shinyjs,useShinyjs)
importFrom(stats,AIC)
importFrom(stats,aggregate)
importFrom(stats,coef)
importFrom(stats,coefficients)
importFrom(stats,cor.test)
importFrom(stats,dchisq)
importFrom(stats,deviance)
importFrom(stats,df.residual)
importFrom(stats,lm)
importFrom(stats,na.exclude)
importFrom(stats,nls)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,time)
importFrom(stringr,str_split)
importFrom(terra,extract)
importFrom(terra,unwrap)
importFrom(terra,vect)
importFrom(terra,wrap)
importFrom(tidyterra,geom_spatvector)
importFrom(utils,data)
importFrom(utils,read.table)
importFrom(utils,write.table)
importFrom(zip,zip)
