## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----eval=FALSE, fig.height=8, fig.width=8------------------------------------
# BioIndex::BioIndex(ta=BioIndex::TA, tb=BioIndex::TB, tc=BioIndex::TC,
#                    sspp="MERLMER",rec_threshold=200,
#                    spaw_threshold=210,sexes="all", depth=c(10,800), GSA=10,
#                    country="all", map_lim=c(13.3,15.2,39.9,41.3),
#                    depth_lines=c(50,200,800), strata=BioIndex::strata_scheme,
#                    stratification_tab = BioIndex::stratification, resolution=1,
#                    buffer=0.1, wd=tempdir(), zip=FALSE, save=FALSE, verbose=TRUE)

