% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dd.distance.R
\name{dd.distance}
\alias{dd.distance}
\title{Estimate hauls distances (decimal degrees)}
\usage{
dd.distance(data, unit = "m", verbose = TRUE)
}
\arguments{
\item{data}{data frame of the hauls data (TA, table A) with coordinates reported as decimal degrees}

\item{unit}{string value indicating the measure unit of the distance. Allowed values: "m" for meters, "km" for kilometers and "NM" for nautical miles.}

\item{verbose}{give verbose output reporting in the output the selected measure unit of the distance.}
}
\value{
The function returns the vector of the distances expressed in the selected measure unit.
}
\description{
Function to estimate the hauls length using TA (table A, hauls data) with coordinates in the decimal degrees format (dd.ddd). The distances could be returned expressed in meters, kilometers and nautical miles.
}
\examples{
data(TA)
ta_dd <- MEDITS.to.dd(TA)
dd.distance(ta_dd, unit = "km", verbose = FALSE)
}
