% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ALK.R
\name{ALK}
\alias{ALK}
\title{ALK (Age Length Key)}
\usage{
ALK(
  ta,
  te,
  sp,
  GSA,
  country = "all",
  nyears = NA,
  wd = NA,
  save = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{ta}{MEDITS or MEDITS-like TA table}

\item{te}{MEDITS or MEDITS-like TE table}

\item{sp}{species RUBIN code (MEDITS format, e.g. "MERLMER")}

\item{GSA}{reference GSA for the analysis}

\item{country}{reference country}

\item{nyears}{number of years of the time series to be considered in the analysis}

\item{wd}{path of the working directory}

\item{save}{boolean. If TRUE the outputs are saved in the local folder}

\item{verbose}{boolean. If TRUE messages are promted in the console}
}
\value{
A \code{data.frame} representing the Age-Length Key.
}
\description{
Computes the Age-Length Key (ALK) from biological samples, a crucial
parameter for converting length distributions into age classes and
analyzing stock growth dynamics.
}
