% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{stratum_0_45}
\alias{stratum_0_45}
\title{stratum_0_45}
\format{
An object of class \code{PackedSpatVector} of length 1.
}
\usage{
stratum_0_45
}
\description{
stratum_0_45
}
\keyword{stratum_0_45}
