% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Lquant.r
\name{Lquant}
\alias{Lquant}
\title{Estimation of L50 and L95}
\usage{
Lquant(lfd, wd = NA, sspp, GSA, save = TRUE, verbose = TRUE)
}
\arguments{
\item{lfd}{data frame of combined LFD}

\item{wd}{working directory}

\item{sspp}{MEDITS code for the selected species}

\item{GSA}{reference area for the analysis}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}

\item{verbose}{boolean. If TRUE messages are reported in the console}
}
\value{
A \code{data.frame} containing the length quantiles (e.g., L95) time series.
}
\description{
Estimation of L50 and L95
}
