% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spearman.R
\name{spearman}
\alias{spearman}
\title{Spearman test}
\usage{
spearman(
  abundance = NA,
  biomass = NA,
  years,
  sspp = NA,
  wd = NA,
  save = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{abundance}{data frame of abundance indices}

\item{biomass}{data frame of biomass indices}

\item{years}{reference years for the analysis}

\item{sspp}{reference species for the analysis}

\item{wd}{path of working directory}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}

\item{verbose}{boolean. If TRUE messages are prompted in the console}
}
\value{
A \code{data.frame} or \code{list} with the results of the Spearman rank correlation test.
}
\description{
Spearman test
}
\examples{
abundance <- data.frame(year = 2010:2020, index = runif(11, 1, 10))
biomass <- data.frame(year = 2010:2020, index = runif(11, 5, 20))
spearman(abundance=abundance, biomass=biomass, years=c(2010,2020), 
         sspp="MERLMER", wd=tempdir(), save=FALSE)
}
