% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{strata_scheme}
\alias{strata_scheme}
\title{stratification scheme}
\format{
An object of class \code{data.frame} with 126 rows and 5 columns.
}
\usage{
strata_scheme
}
\description{
Stratification scheme adopted in the bottom trawl demersal survey (e.g. MEDITS survey).
}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}
\keyword{stratification}
