% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/index_spawn.r
\name{index_spawn}
\alias{index_spawn}
\title{Estimation of abundance indices for spawners (females)}
\usage{
index_spawn(
  mTATB,
  mTATC,
  GSA,
  country,
  depth_range,
  cutoff,
  stratification,
  wd = NA,
  save = TRUE
)
}
\arguments{
\item{mTATB}{data frame}

\item{mTATC}{data frame}

\item{GSA}{reference GSA for the analysis}

\item{country}{vector of reference countries for the analysis}

\item{depth_range}{range of depth strata to perform the analysis (min, max)}

\item{cutoff}{cutoff value for splitting spawner portion of population (reported in mm)}

\item{stratification}{data frame of strata surface area}

\item{wd}{working directory}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}
}
\value{
A \code{data.frame} containing the abundance and biomass indices for the spawner fraction.
}
\description{
Calculates abundance and biomass indices focused on the adult fraction
(spawners, using the user-defined cutoff value), an essential indicator
for evaluating the self-renewal capacity of the population.
}
\examples{
data(TA)
data(TB)
data(TC)
data(stratification)
m <- merge_TATBTC(TA[TA$AREA == 10, ], TB[TB$AREA == 10, ], TC[TC$AREA == 10, ],
                  species = "MERLMER", country = "all", verbose = FALSE)
index_spawn(m[[1]], m[[2]], GSA = 10, country = "all", depth_range = c(10, 800),
            cutoff = 200, stratification = stratification, save = FALSE)
}
