% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hauls_position.r
\name{hauls_position}
\alias{hauls_position}
\title{Plot of hauls time series}
\usage{
hauls_position(
  mTATB,
  country = "all",
  map_lim,
  depth_lines,
  buffer = 0,
  res = NA,
  wd = NA,
  save = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{mTATB}{data frame}

\item{country}{country code as reported in MEDITS format. "all" code to perform the analysis on all the countries of the same GSA}

\item{map_lim}{coordinates limits for the plotted map}

\item{depth_lines}{vactor of three depth bathymetrical lines to be plotted}

\item{buffer}{buffer to the coordinate limits in map units}

\item{res}{resolution of the bathymetrical lines}

\item{wd}{working directory}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}

\item{verbose}{boolean. If TRUE messages are reported in the console}
}
\value{
A \code{ggplot} object representing the spatial distribution of hauls.
}
\description{
This function generates a spatial plot of haul positions over time, displaying haul labels and bathymetric lines over a customizable map extent.
If no resolution is specified (res = NA), the function works offline and uses an internal bathymetry dataset (med_bathy) covering the Mediterranean and Black Sea, reducing the computational time.
}
\examples{
\donttest{
# Use internal data
data(TA)
data(TB)
# Create a merged dataset for GSA 10 and year 2016
mTATB <- merge_TATB(TA[TA$AREA == 10 & TA$YEAR == 2016, ],
                    TB[TB$AREA == 10 & TB$YEAR == 2016, ],
                    "MERLMER")
hauls_position(mTATB, country = "all", map_lim = NA, depth_lines = c(200, 500, 800), save = FALSE)
}
}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}
