% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MIW.r
\name{MIW}
\alias{MIW}
\title{Estimation of Mean Individual Weight (MIW) time series}
\usage{
MIW(
  mTATB,
  GSA,
  country = "all",
  depth_range,
  strata_scheme,
  stratification,
  wd = NA,
  save = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{mTATB}{data frame of the merged TA and TB}

\item{GSA}{reference GSA for the analysis}

\item{country}{reference countries in the GSA for the analysis}

\item{depth_range}{range of depth strata to perform the analysis (min, max)}

\item{strata_scheme}{data frame of the stratification scheme}

\item{stratification}{data frame of strata surface area}

\item{wd}{working directory}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}

\item{verbose}{boolean. If TRUE messages are reported in the console}
}
\value{
A \code{data.frame} containing the Mean Individual Weight (MIW) time series.
}
\description{
Calculates the Mean Individual Weight (MIW) time series, providing a
summary indicator of the average size within the captured population
and potential shifts due to fishing pressure.
}
\examples{
data(TA)
data(TB)
# Create a merged dataset for GSA 10
mTATB <- merge_TATB(TA[TA$AREA==10,], TB[TB$AREA==10,], "MERLMER")
MIW(mTATB, GSA=10, country="all", depth_range=c(10,800),
    strata_scheme=BioIndex::strata_scheme,
    stratification=BioIndex::stratification, wd=tempdir(), save=FALSE)
}
