% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{TC}
\alias{TC}
\title{TC table example}
\format{
An object of class \code{data.frame} with 11185 rows and 22 columns.
}
\usage{
TC
}
\description{
TC table example
}
\keyword{MEDITS}
\keyword{TC}
