% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{TB}
\alias{TB}
\title{TB table example}
\format{
An object of class \code{data.frame} with 3059 rows and 19 columns.
}
\usage{
TB
}
\description{
TB table example
}
\keyword{MEDITS}
\keyword{TB}
