% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spear.r
\name{spear}
\alias{spear}
\title{Spearman test for timeseries}
\usage{
spear(x)
}
\arguments{
\item{x}{time series}
}
\value{
A \code{data.frame} representing the Spearman correlation coefficient and related statistics.
}
\description{
Spearman test for timeseries
}
\examples{
x <- runif(10, 1, 100)
spear(x)
}
