% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{TA}
\alias{TA}
\title{TA table example}
\format{
An object of class \code{data.frame} with 100 rows and 43 columns.
}
\usage{
TA
}
\description{
TA table example
}
\keyword{MEDITS}
\keyword{TA}
