% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indices_ts.R
\name{indices_ts}
\alias{indices_ts}
\title{Estimation of abundance and biomass indices}
\usage{
indices_ts(
  mTATB,
  GSA,
  country = "all",
  depth_range,
  strata_scheme,
  stratification,
  wd = NA,
  save = TRUE
)
}
\arguments{
\item{mTATB}{data frame}

\item{GSA}{reference GSA for the analysis}

\item{country}{reference countries in the GSA for the analysis}

\item{depth_range}{range of depth strata to perform the analysis (min, max)}

\item{strata_scheme}{data frame of the stratification scheme}

\item{stratification}{data frame of strata surface area}

\item{wd}{working directory}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}
}
\value{
A \code{list} of \code{ggplot} objects representing the time series of calculated indices.
}
\description{
Computes time series of stratified abundance and biomass indices,
serving as the primary tool for monitoring temporal trends in stock status.
}
\examples{
data(TA)
data(TB)
# Create a merged dataset for GSA 10
mTATB <- merge_TATB(TA[TA$AREA==10,], TB[TB$AREA==10,], "MERLMER")
# Run indices_ts
indices_ts(mTATB, GSA=10, country="all", depth_range=c(10,800),
           strata_scheme=BioIndex::strata_scheme,
           stratification=BioIndex::stratification, wd=tempdir(), save=FALSE)
}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}
