% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_coordinates.r
\name{convert_coordinates}
\alias{convert_coordinates}
\title{MEDITS coordinates in decimal degrees}
\usage{
convert_coordinates(Data)
}
\arguments{
\item{Data}{data frame of TA table}
}
\value{
the function return the same data frame with the coordinates converted in the decimal degrees format
}
\description{
The function returns the data frame of the TA table with the coordinates expressed as decimal degrees.
}
\examples{
data(TA)
convert_coordinates(TA)
}
