% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quant.r
\name{quant}
\alias{quant}
\title{Quantile estimation}
\usage{
quant(weighted, qlin = 0.95)
}
\arguments{
\item{weighted}{LFD data.frame}

\item{qlin}{reference quantile for the analysis}
}
\value{
A \code{numeric} value or vector containing the requested quantiles.
}
\description{
Quantile estimation
}
