% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{TA_cols}
\alias{TA_cols}
\title{TA table headings}
\format{
An object of class \code{character} of length 22.
}
\usage{
TA_cols
}
\description{
TA table headings
}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}
\keyword{TA}
