% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Bubble_plot_by_haul_indexes.r
\name{bubble_plot_by_haul_indexes}
\alias{bubble_plot_by_haul_indexes}
\title{Bubble plot of abundance and biomass indices by haul}
\usage{
bubble_plot_by_haul_indexes(
  mTATB,
  map_lim,
  depth_lines,
  buffer = 0,
  res = NA,
  wd = NA,
  save = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{mTATB}{data frame}

\item{map_lim}{coordinates limits for the plotted map}

\item{depth_lines}{vector of three depth bathymetrical lines to be plotted}

\item{buffer}{buffer to the coordinate limits in map units}

\item{res}{resolution of the bathymetrical lines}

\item{wd}{working directory}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}

\item{verbose}{boolean. If TRUE a message is printed}
}
\value{
A \code{ggplot} object representing the bubble plot of abundance or biomass by hauls.
}
\description{
The function generates bubble plot of abundance and biomass indices by haul.
If no resolution is specified (res = NA), the function works offline and uses an internal bathymetry dataset (med_bathy) covering the Mediterranean and Black Sea, reducing the computational time.
}
\examples{
\donttest{
# Use internal data
data(TA)
data(TB)
# Create a merged dataset for GSA 10 and year 2016
mTATB <- merge_TATB(TA[TA$AREA == 10 & TA$YEAR == 2016, ],
                    TB[TB$AREA == 10 & TB$YEAR == 2016, ],
                    "MERLMER")
bubble_plot_by_haul_indexes(mTATB, map_lim = NA, depth_lines = c(200, 500, 800), save = FALSE)
}
}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}
