% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_hauls_TBTA.r
\name{check_hauls_TBTA}
\alias{check_hauls_TBTA}
\title{Check hauls TB TA (RoME)}
\usage{
check_hauls_TBTA(DataTA, DataTB, year, wd = NA, suffix = NA)
}
\arguments{
\item{DataTA}{MEDITS TA table}

\item{DataTB}{MEDITS TB table}

\item{year}{reference year for the analysis}

\item{wd}{working directory}

\item{suffix}{name of the log file}
}
\value{
\code{TRUE} if the validation passes, \code{FALSE} otherwise.
}
\description{
Check if all the hauls in TB are in TA.
}
\note{
This function is an internal routine based on \strong{RoME version 0.2.3}.
It is provided within \code{BioIndex} to ensure the package remains
functional and self-sufficient for data validation.
}
\examples{
data(TA)
data(TB)
check_hauls_TBTA(TA, TB, year = 2007)
}
