% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ALKf.R
\name{ALKf}
\alias{ALKf}
\title{Estimation of ALK}
\usage{
ALKf(te, sp, GEAR, GSA, country = NA, years = 5, wd = NA, save = TRUE)
}
\arguments{
\item{te}{MEDITS or MEDITS-like TE table}

\item{sp}{species RUBIN code (MEDITS format, e.g. "MERLMER")}

\item{GEAR}{type of gear reported in the corresponding TA file}

\item{GSA}{reference GSA for the analysis}

\item{country}{reference country}

\item{years}{number of years to be considered in the analysis}

\item{wd}{path of the working directory}

\item{save}{boolean. If TRUE the outputs are saved in the local folder}
}
\value{
A \code{data.frame} representing the Age-Length Key for females.
}
\description{
Estimation of ALK
}
