% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{stratification}
\alias{stratification}
\title{stratification}
\format{
An object of class \code{data.frame} with 276 rows and 6 columns.
}
\usage{
stratification
}
\description{
Data frame containing the surface area relative to the depth strata adopted in the stratification scheme (strata_scheme) of the demersal surveys (e.g. MEDITS survey).
}
\keyword{MEDITS}
\keyword{stratification}
