% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BioIndex.R
\name{BioIndex}
\alias{BioIndex}
\title{Main function to perform BioIndex analysis}
\usage{
BioIndex(
  ta,
  tb,
  tc,
  sspp,
  rec_threshold,
  spaw_threshold,
  haul_threshold = 30,
  sexes = "all",
  depth,
  GSA,
  country = "all",
  map_lim,
  depth_lines = c(10, 200, 800),
  strata = BioIndex::strata_scheme,
  stratification_tab = BioIndex::stratification,
  resolution = NA,
  buffer = 0.1,
  wd = NA,
  zip = TRUE,
  save = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{ta}{data frame of the TA table in the MEDITS format}

\item{tb}{data frame of the TB table in the MEDITS format}

\item{tc}{data frame of the TC table in the MEDITS format}

\item{sspp}{reference species for the analysis}

\item{rec_threshold}{cutoff threshold for recruits (reported in mm)}

\item{spaw_threshold}{cutoff threshold for spawners  (reported in mm)}

\item{haul_threshold}{minimum number of individuals to be used in estimation of the spatial indicators}

\item{sexes}{reference sex for the analysis}

\item{depth}{reference depth range}

\item{GSA}{reference GSA for the analysis}

\item{country}{reference country}

\item{map_lim}{coordinates limits for the maps}

\item{depth_lines}{depth contours to be plotted in the maps (3 values allowed, e.g c(50,200,800))}

\item{strata}{data frame of the reference strata for the study area}

\item{stratification_tab}{data frame of the stratification scheme}

\item{resolution}{resolution of the depth line}

\item{buffer}{buffer around the map}

\item{wd}{path of the working directory}

\item{zip}{boolean. If TRUE the results are stored in a zip file into the working directory}

\item{save}{boolean. If TRUE the results are stored in the working directory}

\item{verbose}{boolean. If TRUE messages are promted in the console}
}
\value{
A \code{list} containing the results of the BioIndex workflow, including data frames and plot objects.
}
\description{
BioIndex is an R package designed to support the standardized analysis of MEDITS trawl survey data and the calculation of biological indicators for selected species and population components.
}
\examples{
\donttest{
BioIndex(ta=TA[TA$YEAR \%in\% c(2015, 2016), ], tb=TB[TB$YEAR \%in\% c(2015, 2016), ],
tc=TC[TC$YEAR \%in\% c(2015, 2016), ], sspp="MERLMER",rec_threshold=200,
spaw_threshold=210,sexes="all", depth=c(10,800), GSA=10, country="all",
map_lim=c(13.3,15.2,39.9,41.3),depth_lines=c(50,200,800),
strata=BioIndex::strata_scheme, stratification_tab =
BioIndex::stratification, resolution=NA, buffer=0.1, wd=tempdir(),
zip=FALSE, save=TRUE, verbose=TRUE)
}

}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}
