% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Bubbleplot_RS_by_hauls.r
\name{bubbleplot_RS_by_hauls}
\alias{bubbleplot_RS_by_hauls}
\title{Bubbleplot of abundance indices for recruits and spawners}
\usage{
bubbleplot_RS_by_hauls(
  mTATC,
  map_range,
  thresh_rec,
  thresh_spaw,
  depths = c(50, 200, 800),
  res = NA,
  buffer = 0.1,
  wd = NA,
  save = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{mTATC}{mTATC table}

\item{map_range}{range of coordinates for the map}

\item{thresh_rec}{threshold value to select recruits data from mTATC table (reported in mm)}

\item{thresh_spaw}{threshold value to select spawners data from mTATC table (reported in mm)}

\item{depths}{three reference bathymetric lines to be plotted in the maps}

\item{res}{resolution of the depth lines}

\item{buffer}{buffer around the map}

\item{wd}{working directory}

\item{save}{boolean. If TRUE the outputs are saved in the local folder}

\item{verbose}{boolean. If TRUE messages are prompted in the console}
}
\value{
A \code{list} containing two \code{ggplot} objects: the bubble plot of recruits (\code{pr}) and spawners (\code{ps}) abundance indices.
}
\description{
The function generates bubbleplots of abundance indices for recruits and spawners
If no resolution is specified (res = NA), the function works offline and uses an internal bathymetry dataset (med_bathy) covering the Mediterranean and Black Sea, reducing the computational time.
}
\examples{
\donttest{
# Create a merged dataset for GSA 10
mTATC <- merge_TATC(TA[TA$AREA==10,], TC[TC$AREA==10,], "MERLMER")
bubbleplot_RS_by_hauls(mTATC, map_range=c(15,21,39,43), thresh_rec=200, thresh_spaw=250)
}
}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}
