% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_BioIndex_app.r
\name{run_BioIndex_app}
\alias{run_BioIndex_app}
\title{launches the embedded Shiny application included in the package.}
\usage{
run_BioIndex_app()
}
\value{
\code{No return value, called for side effects} (launches the Shiny application).
}
\description{
launches the embedded Shiny application included in the package.
}
\examples{
if (interactive()) {
  run_BioIndex_app()
}
}
\author{
Walter Zupa \email{zupa@fondazionecoispa.org}
}
