% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.r
\docType{data}
\name{stratification_rapana}
\alias{stratification_rapana}
\title{stratification (rapa whelk)}
\format{
An object of class \code{data.frame} with 7 rows and 6 columns.
}
\usage{
stratification_rapana
}
\description{
Data frame containing the surface area relative to the depth strata adopted in the stratification scheme (strata_scheme) of the rapa whelk surveys (e.g. Black Sea beam trawl survey).
}
\keyword{Black}
\keyword{Sea}
\keyword{rapa}
\keyword{stratification}
\keyword{whelk}
