% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LW.R
\name{LW}
\alias{LW}
\title{LW (Length-Weight relationship)}
\usage{
LW(
  ta,
  te,
  sp,
  GSA,
  country = "all",
  nyears = NA,
  wd = NA,
  save = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{ta}{MEDITS or MEDITS-like TA table}

\item{te}{MEDITS or MEDITS-like TE table}

\item{sp}{species RUBIN code (MEDITS format, e.g. "MERLMER")}

\item{GSA}{reference GSA for the analysis}

\item{country}{reference country}

\item{nyears}{number of years of the time series to be considered in the analysis}

\item{wd}{path of the working directory}

\item{save}{boolean. If TRUE the outputs are saved in the local folder}

\item{verbose}{boolean. If TRUE messages are promted in the console}
}
\value{
A \code{data.frame} containing the length-weight relationship parameters and statistics.
}
\description{
Estimates the length-weight relationship for the selected species,
a fundamental biological indicator to assess fish condition and convert
numerical length estimates into biomass equivalents.
}
