% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MEDITS.distance.R
\name{MEDITS.distance}
\alias{MEDITS.distance}
\title{Estimation of haul distance}
\usage{
MEDITS.distance(data, unit = "m", verbose = TRUE)
}
\arguments{
\item{data}{data frame containing the hauls data (TA, table A).}

\item{unit}{string value indicating the measure unit of the distance. Allowed values: "m" for meters, "km" for kilometers and "NM" for nautical miles.}

\item{verbose}{give verbose output reporting in the output the selected measure unit of the distance.}
}
\value{
The function returns the vector of the distances expressed in the selected measure unit.
}
\description{
Estimation of haul distance
}
\examples{
data(TA)
MEDITS.distance(TA, unit = "m", verbose = FALSE)
}
