% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MEDITS.to.dd.R
\name{MEDITS.to.dd}
\alias{MEDITS.to.dd}
\title{Conversion of MEDITS format coordinates in decimal degrees format}
\usage{
MEDITS.to.dd(data)
}
\arguments{
\item{data}{data frame of the hauls data (TA, table A) in MEDITS format}
}
\value{
The function returns the data frame of the TA table with the coordinates expressed as decimal degrees
}
\description{
Conversion of MEDITS format coordinates in decimal degrees format
}
\examples{
data(TA)
MEDITS.to.dd(TA)
}
