% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlayGrid.r
\name{overlayGrid}
\alias{overlayGrid}
\title{Overlay mTATB and mTATC on GFCM spatial grid}
\usage{
overlayGrid(
  mTATB,
  mTATC,
  GSA = NA,
  country = "all",
  wd = NA,
  save = TRUE,
  verbose = FALSE
)
}
\arguments{
\item{mTATB}{data frame of the merged TA and TB}

\item{mTATC}{data frame of the merged TA and TC}

\item{GSA}{reference GSA for the analysis}

\item{country}{reference countries for the analysis}

\item{wd}{working directory used to save results}

\item{save}{boolean. If TRUE the outputs are saved in the local folder}

\item{verbose}{boolean. If TRUE messages are prompted in the console}
}
\value{
A \code{list} or \code{data.frame} containing the merged biological data mapped to the spatial grid.
}
\description{
Overlays survey haul data onto a standard spatial grid (e.g., GFCM grid),
harmonizing biological information with a consistent geographic structure
for standardized spatial analysis.
}
\examples{
# Use internal data
data(TA)
data(TB)
data(TC)
# Create a merged dataset for GSA 10
m <- merge_TATBTC(TA[TA$AREA==10,], TB[TB$AREA==10,], TC[TC$AREA==10,], species="MERLMER")
mTATB <- m[[1]]
mTATC <- m[[2]]
overlayGrid(mTATB, mTATC, GSA=10, save=FALSE)
}
