% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/index_ts_M.r
\name{index_ts_M}
\alias{index_ts_M}
\title{Estimation of abundance indices for males}
\usage{
index_ts_M(
  mTATB,
  GSA,
  country_analysis,
  depth_range,
  strata_scheme,
  stratification,
  wd = NA,
  save = TRUE
)
}
\arguments{
\item{mTATB}{data frame}

\item{GSA}{reference GSA for the analysis}

\item{country_analysis}{vector of reference countries for the analysis}

\item{depth_range}{range of depth strata to perform the analysis (min, max)}

\item{strata_scheme}{data frame of the stratification scheme}

\item{stratification}{data frame of strata surface area}

\item{wd}{working directory}

\item{save}{boolean. If TRUE the plot is saved in the user defined working directory (wd)}
}
\value{
A \code{data.frame} containing the time series of indices specifically for males.
}
\description{
Estimation of abundance indices for males
}
\examples{
data(TA)
data(TB)
data(TC)
data(strata_scheme)
data(stratification)
m <- merge_TATBTC(TA[TA$AREA == 10, ], TB[TB$AREA == 10, ], TC[TC$AREA == 10, ],
                  species = "MERLMER", country = "all", verbose = FALSE)
strata_scheme_g10 <- strata_scheme[strata_scheme$GSA == 10 & strata_scheme$COUNTRY == "ITA", ]
index_ts_M(m[[1]], GSA = 10, country_analysis = "all", depth_range = c(10, 800),
           strata_scheme = strata_scheme_g10, stratification = stratification, save = FALSE)
}
