% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dd.to.MEDITS.R
\name{dd.to.MEDITS}
\alias{dd.to.MEDITS}
\title{Conversion of decimal degrees coordinates in MEDITS format}
\usage{
dd.to.MEDITS(data)
}
\arguments{
\item{data}{data frame of the hauls data (TA, table A) in MEDITS format}
}
\value{
The function returns the data frame of the TA (table A) reporting the coordinates in MEDITS format.
}
\description{
Conversion of decimal degrees coordinates in MEDITS format
}
\examples{
data(TA)
ta_dd <- MEDITS.to.dd(TA)
dd.to.MEDITS(ta_dd)
}
